import sys
import platform
import serial
import threading
import time
import re

def array2hexlist(list):
    ret = ""
    for b in list:
        ret += "%0.2X " % ord(b)  
    return ret[:-1]


class MinT10(object):
    PORT = '/dev/ttyUSB0' #linux
    if platform.system() == "Windows":
        PORT = 'COM25'

    def __init__(self):
        self.serport = serial.Serial(self.PORT, 9600, bytesize = serial.SEVENBITS, parity = serial.PARITY_EVEN, stopbits = serial.STOPBITS_ONE, timeout=None)

        #self.thread_alive = True
        #self.thread_reader = threading.Thread(target=self.serialreaderthread)
        #self.thread_reader.setDaemon(True)
        #self.thread_reader.setName('serial port reader')
        #self.thread_reader.start()
    

    def __exit__(self):
        self.thread_alive = False
        #self.thread_reader.exit()

    def serialreaderthread(self):
        
        while self.thread_alive:
            c = ord(self.serport.read())
            print("{0:2x} {1}".format(c, chr(c)))


    def command(self, receptor, command, data4):
        if receptor > 99 or command > 99:
            return False

        receptor = "{:0>2d}".format(receptor)
        command = "{:0>2d}".format(command)

        data4 =  "{: <4s}".format(str(data4))

        out = chr(0x02) #STX
        out += receptor + command
        out += data4
        out += chr(0x03) #ETX
        bcc = self.calc_bcc(receptor + command + data4 + chr(0x03))
        out += "{:0>2x}".format(ord(bcc))
        out += "\r\n" #CRLF
 
        #print("PC -> T-10: " + array2hexlist(out))
        out2 = str(out)
        self.serport.write(out)
        return True

    def SetPcMode(self):
        self.command(0, 54, 1)

    def ReadIlluminance(self):
        self.serport.flushInput()
        self.command(0, 10, "0200")
        time.sleep(0.08);
        
        data = self.serport.readline()

        
        p = re.compile(ur'\x02(?P<Receptor>[0-9]{2})(?P<Command>[0-9]{2})(?P<Status>[ 0-9]{4})(?P<Luminance>[+\-=][ 0-9]{5})(?P<Deltae>[+\-=][ 0-9]{5})(?P<Percent>[+\-=][ 0-9]{5})\x03(?P<BCC>[0-9A-F]{2})\r\n')
        foo = re.search(p, data)

        luminance = self.ParseLuminance(foo.group('Luminance'))
        return luminance

    def ParseLuminance(self, data):
        factor = +1
        if data[0] == '-':
            factor = -1

        value = int(data[1:5])
        factor *= pow(10, int(data[5]) - 4)

        return float(value) * factor

    
    def calc_bcc(self, content):
        out = 0
        for byte in content:
            out = out ^ ord(byte)

        return chr(out)


