﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;
using System.Threading;
using System.Text.RegularExpressions;

namespace ALScalib
{
    public class MinoltaT10
    {

        SerialPort port = null;

        public MinoltaT10(string portname)
        {
            port = new SerialPort(portname, 9600, Parity.Even, 7, StopBits.One);
            port.Open();
        }

        public bool SendCommand(int receptor, int command, int data4)
        {
            if (receptor > 99 || command > 99 || data4 > 9999)
            {
                return false;
            }

            string data = string.Format("{0:00}{1:00}{2,-4}\x03", receptor, command, data4);

            char bcc = calc_bcc(data);
            data = string.Format("\x02{0}{1:X2}\r\n", data, (int)bcc);

            port.Write(data);

            return true;
        }

        public bool SendCommand(int receptor, int command, string data4)
        {
            if (receptor > 99 || command > 99 || data4.Length > 4)
            {
                return false;
            }

            string data = string.Format("{0:00}{1:00}{2}\x03", receptor, command, data4.PadLeft(4, '0'));

            char bcc = calc_bcc(data);
            data = string.Format("\x02{0}{1:X2}\r\n", data, (int)bcc);

            port.Write(data);

            string data_hex = StringToHex(data);

            return true;
        }

        private string StringToHex(string hexstring)
        {
            var sb = new StringBuilder();
            foreach (char t in hexstring)
                sb.Append(Convert.ToInt32(t).ToString("X2") + " ");
            return sb.ToString();
        }

        public void PCMode(int receptor)
        {
            SendCommand(receptor, 54, 1);
            Thread.Sleep(100);
        }

        public double ReadIlluminance(int receptor)
        {

            SendCommand(receptor, 10, "200");
            string crap = port.ReadExisting();
            Thread.Sleep(100);

            string rx = port.ReadExisting();
            string rxhex = StringToHex(rx);
            Regex regex = new Regex(@"\x02(?<receptor>[0-9]{2})(?<command>[0-9]{2})(?<status>[ 0-9]{4})(?<luminance>[+\-=][ 0-9]{5})(?<deltae>[+\-=][ 0-9]{5})(?<percent>[+\-=][ 0-9]{5})\x03(?<bcc>[0-9A-F]{2})", RegexOptions.CultureInvariant);
            //


            Match m = regex.Match(rx);

            if (m.Success == false)
            {
                return double.NaN;

                
            }

            string rereceptor = m.Groups["receptor"].Value;
            string command = m.Groups["command"].Value;
            
            string status = m.Groups["status"].Value;
            string luminance = m.Groups["luminance"].Value;
            string deltae = m.Groups["deltae"].Value;
            string percent = m.Groups["percent"].Value;
            string bcc = m.Groups["bcc"].Value;

            double factor = 1;
            if (luminance[0] == '-')
            {
                factor = -1;
            }

            double lum = (double)int.Parse(luminance.Substring(1, 4));

            factor *= Math.Pow(10, int.Parse(luminance.Substring(5, 1))-4);

            return lum * factor;


        }

        private char calc_bcc(string data)
        {
            char retval = '\0';
            for (int i = 0; i < data.Length; i++)
            {
                retval ^= data[i];
            }
            return retval;
        }


    }
}
